from django.utils.functional import cached_property, lazy
from wagtail.core.blocks import ChooserBlock
from wagtail.core.utils import resolve_model_string
from instance_selector.widgets import InstanceSelectorWidget
from instance_selector.registry import registry


class InstanceSelectorBlock(ChooserBlock):
    class Meta:
        icon = "placeholder"

    def __init__(self, target_model, **kwargs):
        super().__init__(**kwargs)

        self._target_model = target_model

        if self.meta.icon == "placeholder":
            # The models/selectors may not have been registered yet, depending upon
            # import orders and things, so get the icon lazily
            self.meta.icon = lazy(self.get_instance_selector_icon, str)

    @cached_property
    def target_model(self):
        return resolve_model_string(self._target_model)

    @cached_property
    def widget(self):
        return InstanceSelectorWidget(self.target_model)

    def get_instance_selector_icon(self):
        instance_selector = registry.get_instance_selector(self.target_model)
        return instance_selector.get_widget_icon()

    def deconstruct(self):
        name, args, kwargs = super().deconstruct()

        if args:
            args = args[1:]  # Remove the args target_model

        kwargs["target_model"] = self.target_model._meta.label_lower
        return name, args, kwargs



import time
from django.http import HttpResponseBadRequest, JsonResponse
from django.core.exceptions import PermissionDenied
from django.template.response import TemplateResponse
from django.utils.text import slugify
from instance_selector.constants import OBJECT_PK_PARAM
from instance_selector.registry import registry


def user_can_access_admin(user):
    if not user:
        return False
    return (
        user.is_superuser or user.is_staff or user.has_perm("wagtailadmin.access_admin")
    )


def instance_selector_embed(request, app_label, model_name):
    if not user_can_access_admin(request.user):
        raise PermissionDenied

    model = registry.get_model(app_label, model_name)
    instance_selector = registry.get_instance_selector(model)
    instance_selector_url = instance_selector.get_instance_selector_url()
    embed_id = slugify("%s-%s-%s" % (app_label, model_name, time.time()))
    embed_url = "%s#instance_selector_embed_id:%s" % (instance_selector_url, embed_id)

    context = {"embed_url": embed_url, "embed_id": embed_id}
    return TemplateResponse(
        request, "instance_selector/instance_selector_embed.html", context
    )


def instance_selector_lookup(request, app_label, model_name):
    if not user_can_access_admin(request.user):
        raise PermissionDenied

    object_pk = request.GET.get(OBJECT_PK_PARAM)
    if not object_pk:
        return HttpResponseBadRequest(
            "Param `%s` does have a value defined" % OBJECT_PK_PARAM
        )

    model = registry.get_model(app_label, model_name)
    instance = model.objects.get(pk=object_pk)
    instance_selector = registry.get_instance_selector(model)

    display_markup = instance_selector.get_instance_display_markup(instance)
    edit_url = instance_selector.get_instance_edit_url(instance)

    return JsonResponse(
        {"display_markup": display_markup, "edit_url": edit_url, "pk": object_pk}
    )



from django.template.loader import render_to_string
from django.urls import reverse
from django.utils.translation import ugettext_lazy as _
from wagtail.admin.widgets import AdminChooser
from instance_selector.constants import OBJECT_PK_PARAM
from instance_selector.registry import registry


class InstanceSelectorWidget(AdminChooser):
    def __init__(self, model, **kwargs):
        self.target_model = model

        model_name = self.target_model._meta.verbose_name
        self.choose_one_text = _("Choose %s") % model_name
        self.choose_another_text = _("Choose another %s") % model_name
        self.link_to_chosen_text = _("Edit this %s") % model_name

        super().__init__(**kwargs)

    def render_html(self, name, value, attrs):
        original_field_html = super().render_html(name, value, attrs)

        instance, value = self.get_instance_and_id(self.target_model, value)

        app_label = self.target_model._meta.app_label
        model_name = self.target_model._meta.model_name
        model = registry.get_model(app_label, model_name)
        instance_selector = registry.get_instance_selector(model)

        embed_url = reverse(
            "wagtail_instance_selector_embed",
            kwargs={"app_label": app_label, "model_name": model_name},
        )
        # We use the input name for the embed id so that wagtail's block code will automatically
        # replace any `__prefix__` substring with a specific id for the widget instance
        embed_id = name
        embed_url += "#instance_selector_embed_id:" + embed_id

        lookup_url = reverse(
            "wagtail_instance_selector_lookup",
            kwargs={"app_label": app_label, "model_name": model_name},
        )

        display_markup = instance_selector.get_instance_display_markup(instance)
        edit_url = instance_selector.get_instance_edit_url(instance)

        return render_to_string(
            "instance_selector/instance_selector_widget.html",
            {
                "name": name,
                "value": value,
                "widget": self,
                "input_id": attrs["id"],
                "widget_id": "%s-instance-selector-widget" % attrs["id"],
                "original_field_html": original_field_html,
                "embed_url": embed_url,
                "embed_id": embed_id,
                "lookup_url": lookup_url,
                "OBJECT_PK_PARAM": OBJECT_PK_PARAM,
                "display_markup": display_markup,
                "edit_url": edit_url,
            },
        )



from setuptools import setup
import instance_selector

setup(
    name="wagtail-instance-selector",
    version=instance_selector.__version__,
    packages=["instance_selector"],
    include_package_data=True,
    description="A widget for Wagtail's admin that allows you to create and select related items",
    long_description="Documentation at https://github.com/ixc/wagtail-instance-selector",
    author="The Interaction Consortium",
    author_email="admins@interaction.net.au",
    url="https://github.com/ixc/wagtail-instance-selector",
)
